@@ -390,6 +390,8 @@ def cast(arg, target_type):
         return arg
     else:
         result = op.to_expr()
+        if not arg.has_name():
+            return result
         try:
             expr_name = ('cast({0}, {1!s})'
                          .format(arg.get_name(),
@@ -1522,7 +1524,7 @@ def _string_contains(arg, substr):
     -------
     contains : boolean
     """
-    return arg.like('%{0}%'.format(substr))
+    return arg.find(substr) >= 0
 
 
 def _string_dunder_contains(arg, substr):
