@@ -42,7 +42,7 @@ class TableNode(Node):
     def get_type(self, name):
         return self.get_schema().get_type(name)
 
-    def to_expr(self):
+    def _make_expr(self):
         return TableExpr(self)
 
     def aggregate(self, this, metrics, by=None, having=None):
@@ -158,7 +158,7 @@ class TableArrayView(ValueNode):
 
         Node.__init__(self, [table])
 
-    def to_expr(self):
+    def _make_expr(self):
         ctype = self.table._get_type(self.name)
         klass = ctype.array_type()
         return klass(self, name=self.name)
@@ -1610,12 +1610,12 @@ class SortKey(ir.Node):
     def root_tables(self):
         return self.expr._root_tables()
 
-    def to_expr(self):
+    def _make_expr(self):
         return ir.SortExpr(self)
 
-    def equals(self, other):
+    def equals(self, other, cache=None):
         return (isinstance(other, SortKey) and
-                self.expr.equals(other.expr) and
+                self.expr.equals(other.expr, cache=cache) and
                 self.ascending == other.ascending)
 
 
@@ -1792,11 +1792,12 @@ class AggregateSelection(object):
     def _attempt_pushdown(self):
         metrics_valid, lowered_metrics = self._pushdown_exprs(self.metrics)
         by_valid, lowered_by = self._pushdown_exprs(self.by)
+        having_valid, lowered_having = self._pushdown_exprs(self.having or None)
 
-        if metrics_valid and by_valid:
+        if metrics_valid and by_valid and having_valid:
             return Aggregation(self.op.table, lowered_metrics,
                                by=lowered_by,
-                               having=self.having,
+                               having=lowered_having,
                                predicates=self.op.predicates,
                                sort_keys=self.op.sort_keys)
         else:
@@ -1859,13 +1860,13 @@ class Aggregation(TableNode, HasSchema):
 
         self.agg_exprs = self._rewrite_exprs(agg_exprs)
 
-        by = by or []
+        by = [] if by is None else by
         self.by = self.table._resolve(by)
 
-        self.having = having or []
+        self.having = [] if having is None else having
 
-        self.predicates = predicates or []
-        sort_keys = sort_keys or []
+        self.predicates = [] if predicates is None else predicates
+        sort_keys = [] if sort_keys is None else sort_keys
         self.sort_keys = [to_sort_key(self.table, k)
                           for k in util.promote_list(sort_keys)]
 
