@@ -58,17 +58,24 @@ class Expr(object):
         else:
             return self._repr()
 
+    def __bool__(self):
+        raise ValueError("The truth value of an Ibis expression is not "
+                         "defined")
+
+    def __nonzero__(self):
+        return self.__bool__()
+
     def _repr(self, memo=None):
         from ibis.expr.format import ExprFormatter
-        return ExprFormatter(self).get_result()
+        return ExprFormatter(self, memo=memo).get_result()
 
     def pipe(self, f, *args, **kwargs):
         """
         Generic composition function to enable expression pipelining
 
         >>> (expr
-             .pipe(f, *args, **kwargs)
-             .pipe(g, *args2, **kwargs2))
+        >>>  .pipe(f, *args, **kwargs)
+        >>>  .pipe(g, *args2, **kwargs2))
 
         is equivalent to
 
@@ -86,9 +93,9 @@ class Expr(object):
         Examples
         --------
         >>> def foo(data, a=None, b=None):
-                pass
+        ...     pass
         >>> def bar(a, b, data=None):
-                pass
+        ...     pass
         >>> expr.pipe(foo, a=5, b=10)
         >>> expr.pipe((bar, 'data'), 1, 2)
 
@@ -159,10 +166,10 @@ class Expr(object):
         except:
             return False
 
-    def equals(self, other):
+    def equals(self, other, cache=None):
         if type(self) != type(other):
             return False
-        return self._arg.equals(other._arg)
+        return self._arg.equals(other._arg, cache=cache)
 
     def _can_compare(self, other):
         return False
@@ -202,26 +209,15 @@ class Node(object):
         return self._repr()
 
     def _repr(self, memo=None):
-        # Quick and dirty to get us started
+        if memo is None:
+            from ibis.expr.format import FormatMemo
+            memo = FormatMemo()
+
         opname = type(self).__name__
         pprint_args = []
 
-        memo = memo or {}
-
-        if id(self) in memo:
-            return memo[id(self)]
-
         def _pp(x):
-            if isinstance(x, Expr):
-                key = id(x.op())
-            else:
-                key = id(x)
-
-            if key in memo:
-                return memo[key]
-            result = _safe_repr(x, memo=memo)
-            memo[key] = result
-            return result
+            return _safe_repr(x, memo=memo)
 
         for x in self.args:
             if isinstance(x, (tuple, list)):
@@ -245,16 +241,30 @@ class Node(object):
             else:
                 yield arg
 
-    def equals(self, other):
+    def equals(self, other, cache=None):
+        if cache is None:
+            cache = {}
+
+        if (self, other) in cache:
+            return cache[(self, other)]
+
+        if id(self) == id(other):
+            cache[(self, other)] = True
+            return True
+
         if type(self) != type(other):
+            cache[(self, other)] = False
             return False
 
         if len(self.args) != len(other.args):
+            cache[(self, other)] = False
             return False
 
         for left, right in zip(self.args, other.args):
-            if not all_equal(left, right):
+            if not all_equal(left, right, cache=cache):
+                cache[(self, other)] = False
                 return False
+        cache[(self, other)] = True
         return True
 
     def is_ancestor(self, other):
@@ -263,7 +273,15 @@ class Node(object):
 
         return self.equals(other)
 
+    _expr_cached = None
+
     def to_expr(self):
+        if self._expr_cached is None:
+            self._expr_cached = self._make_expr()
+        return self._expr_cached
+        # return self._make_expr()
+
+    def _make_expr(self):
         klass = self.output_type()
         return klass(self)
 
@@ -275,17 +293,17 @@ class Node(object):
         raise NotImplementedError
 
 
-def all_equal(left, right):
+def all_equal(left, right, cache=None):
     if isinstance(left, list):
         if not isinstance(right, list):
             return False
         for a, b in zip(left, right):
-            if not all_equal(a, b):
+            if not all_equal(a, b, cache=cache):
                 return False
         return True
 
     if hasattr(left, 'equals'):
-        return left.equals(right)
+        return left.equals(right, cache=cache)
     else:
         return left == right
     return True
@@ -310,6 +328,9 @@ class ValueNode(Node):
     def resolve_name(self):
         raise com.ExpressionError('Expression is not named: %s' % repr(self))
 
+    def has_resolved_name(self):
+        return False
+
 
 class TableColumn(ValueNode):
 
@@ -321,7 +342,9 @@ class TableColumn(ValueNode):
         Node.__init__(self, [name, table_expr])
 
         if name not in table_expr.schema():
-            raise KeyError("'{0}' is not a field".format(name))
+            raise com.IbisTypeError(
+                "'{0}' is not a field in {1}".format(name, table_expr.columns)
+            )
 
         self.name = name
         self.table = table_expr
@@ -332,10 +355,13 @@ class TableColumn(ValueNode):
     def resolve_name(self):
         return self.name
 
+    def has_resolved_name(self):
+        return True
+
     def root_tables(self):
         return self.table._root_tables()
 
-    def to_expr(self):
+    def _make_expr(self):
         ctype = self.table._get_type(self.name)
         klass = ctype.array_type()
         return klass(self, name=self.name)
@@ -400,7 +426,7 @@ class Literal(ValueNode):
     def args(self):
         return [self.value]
 
-    def equals(self, other):
+    def equals(self, other, cache=None):
         if not isinstance(other, Literal):
             return False
         return (isinstance(other.value, type(self.value)) and
@@ -419,6 +445,8 @@ class Literal(ValueNode):
             klass = StringScalar
         elif isinstance(self.value, datetime.datetime):
             klass = TimestampScalar
+        elif isinstance(self.value, datetime.date):
+            klass = DateScalar
         else:
             raise com.InputTypeError(self.value)
 
@@ -452,14 +480,14 @@ class ValueExpr(Expr):
         Expr.__init__(self, arg)
         self._name = name
 
-    def equals(self, other):
+    def equals(self, other, cache=None):
         if not isinstance(other, ValueExpr):
             return False
 
         if self._name != other._name:
             return False
 
-        return Expr.equals(self, other)
+        return Expr.equals(self, other, cache=cache)
 
     def type(self):
         import ibis.expr.datatypes as dt
@@ -474,6 +502,11 @@ class ValueExpr(Expr):
         rule = ImplicitCast(self.type(), self._implicit_casts)
         return rule.can_cast(typename)
 
+    def has_name(self):
+        if self._name is not None:
+            return True
+        return self.op().has_resolved_name()
+
     def get_name(self):
         if self._name is not None:
             # This value has been explicitly named
@@ -684,7 +717,7 @@ class TableExpr(Expr):
 
         return self.projection([self, expr])
 
-    def group_by(self, by):
+    def group_by(self, by=None, **additional_grouping_expressions):
         """
         Create an intermediate grouped table expression, pending some group
         operation to be applied with it.
@@ -693,12 +726,19 @@ class TableExpr(Expr):
         --------
         x.group_by([b1, b2]).aggregate(metrics)
 
+        Notes
+        -----
+        group_by and groupby are equivalent, with `groupby` being provided for
+        ease-of-use for pandas users.
+
         Returns
         -------
         grouped_expr : GroupedTableExpr
         """
         from ibis.expr.groupby import GroupedTableExpr
-        return GroupedTableExpr(self, by)
+        return GroupedTableExpr(self, by, **additional_grouping_expressions)
+
+    groupby = group_by
 
 
 # -----------------------------------------------------------------------------
@@ -808,6 +848,30 @@ class DecimalValue(NumericValue):
         return constructor
 
 
+class DateValue(AnyValue):
+
+    _typename = 'date'
+
+    def _can_implicit_cast(self, arg):
+        op = arg.op()
+        if isinstance(op, Literal):
+            try:
+                import pandas as pd
+                pd.Timestamp(op.value)
+                return True
+            except ValueError:
+                return False
+        return False
+
+    def _can_compare(self, other):
+        return isinstance(other, DateValue)
+
+    def _implicit_cast(self, arg):
+        # assume we've checked this is OK at this point...
+        op = arg.op()
+        return DateScalar(op)
+
+
 class TimestampValue(AnyValue):
 
     _typename = 'timestamp'
@@ -843,6 +907,10 @@ class NullScalar(NullValue, ScalarExpr):
     pass
 
 
+class NullArray(ArrayExpr, NullValue):
+    pass
+
+
 class BooleanScalar(ScalarExpr, BooleanValue):
     pass
 
@@ -907,6 +975,14 @@ class StringArray(ArrayExpr, StringValue):
     pass
 
 
+class DateScalar(ScalarExpr, DateValue):
+    pass
+
+
+class DateArray(ArrayExpr, DateValue):
+    pass
+
+
 class TimestampScalar(ScalarExpr, TimestampValue):
     pass
 
@@ -1065,13 +1141,13 @@ class NullLiteral(ValueNode):
     """
 
     def __init__(self):
-        pass
+        self.value = None
 
     @property
     def args(self):
-        return [None]
+        return [self.value]
 
-    def equals(self, other):
+    def equals(self, other, cache=None):
         return isinstance(other, NullLiteral)
 
     def output_type(self):
@@ -1102,7 +1178,7 @@ class ValueList(ValueNode):
     def root_tables(self):
         return distinct_roots(*self.values)
 
-    def to_expr(self):
+    def _make_expr(self):
         return ListExpr(self)
 
 
