@@ -80,12 +80,10 @@ class ImpalaConnection(object):
 
         self.options = {}
 
-        self.connection_pool = queue.Queue(pool_size)
-        self.connection_pool_size = 0
         self.max_pool_size = pool_size
+        self._connections = None
 
-        self._connections = weakref.WeakValueDictionary()
-
+        self.reset_connection_pool()
         self.ping()
 
     def set_options(self, options):
@@ -95,8 +93,15 @@ class ImpalaConnection(object):
         """
         Close all open Impyla sessions
         """
-        for k, con in self._connections.items():
-            con.close()
+        self.reset_connection_pool()
+
+    def reset_connection_pool(self):
+        if self._connections is not None:
+            for k, con in self._connections.items():
+                con.close()
+        self._connections = weakref.WeakValueDictionary()
+        self.connection_pool = queue.Queue(self.max_pool_size)
+        self.connection_pool_size = 0
 
     def set_database(self, name):
         self.database = name
@@ -146,6 +151,7 @@ class ImpalaConnection(object):
             if (cur.database != self.database or
                     cur.options != self.options):
                 cur = self._new_cursor()
+            cur.released = False
 
             return cur
         except queue.Empty:
@@ -200,7 +206,14 @@ class ImpalaCursor(object):
             self._cursor.close()
         except HS2Error as e:
             # connection was closed elsewhere
-            if 'invalid session' not in e.args[0].lower():
+            already_closed_messages = [
+                'invalid query handle',
+                'invalid session',
+            ]
+            for message in already_closed_messages:
+                if message in e.args[0].lower():
+                    break
+            else:
                 raise
 
     def __enter__(self):
