@@ -355,3 +355,22 @@ LIMIT 10"""
         left.join(right, ['id'])
         left.join(right, ['id', 'name'])
         left.join(right, ['id', 'files'])
+
+    def test_rerelease_cursor(self):
+        con = connect_test(self.env)
+
+        with con.raw_sql('select 1', True) as cur1:
+            pass
+
+        cur1.release()
+
+        with con.raw_sql('select 1', True) as cur2:
+            pass
+
+        cur2.release()
+
+        with con.raw_sql('select 1', True) as cur3:
+            pass
+
+        assert cur1 == cur2
+        assert cur2 == cur3
