@@ -12,12 +12,16 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+import numbers
 import operator
 import six
 
 import sqlalchemy as sa
 import sqlalchemy.sql as sql
 
+from sqlalchemy.sql.elements import Over as _Over
+from sqlalchemy.ext.compiler import compiles as sa_compiles
+
 from ibis.client import SQLClient, AsyncQuery, Query
 from ibis.sql.compiler import Select, Union, TableSetFormatter
 import ibis.common as com
@@ -31,34 +35,43 @@ import ibis
 
 
 _ibis_type_to_sqla = {
-    dt.Int8: sa.types.SmallInteger,
-    dt.Int16: sa.types.SmallInteger,
-    dt.Int32: sa.types.Integer,
-    dt.Int64: sa.types.BigInteger,
+    dt.Int8: sa.SmallInteger,
+    dt.Int16: sa.SmallInteger,
+    dt.Int32: sa.Integer,
+    dt.Int64: sa.BigInteger,
 
     # Mantissa-based
-    dt.Float: sa.types.Float(precision=24),
-    dt.Double: sa.types.Float(precision=53),
+    dt.Float: sa.Float(precision=24),
+    dt.Double: sa.Float(precision=53),
 
-    dt.Boolean: sa.types.Boolean,
+    dt.Boolean: sa.Boolean,
 
-    dt.String: sa.types.String,
+    dt.String: sa.String,
 
-    dt.Timestamp: sa.types.DateTime,
+    dt.Date: sa.Date,
 
-    dt.Decimal: sa.types.NUMERIC,
+    dt.Timestamp: sa.DateTime,
+
+    dt.Decimal: sa.NUMERIC,
 }
 
 _sqla_type_mapping = {
-    sa.types.SmallInteger: dt.Int16,
-    sa.types.INTEGER: dt.Int64,
-    sa.types.BOOLEAN: dt.Boolean,
-    sa.types.BIGINT: dt.Int64,
-    sa.types.FLOAT: dt.Double,
-    sa.types.REAL: dt.Double,
+    sa.SmallInteger: dt.Int16,
+    sa.SMALLINT: dt.Int16,
+    sa.Integer: dt.Int32,
+    sa.INTEGER: dt.Int32,
+    sa.BigInteger: dt.Int64,
+    sa.BIGINT: dt.Int64,
+    sa.Boolean: dt.Boolean,
+    sa.BOOLEAN: dt.Boolean,
+    sa.FLOAT: dt.Double,
+    sa.REAL: dt.Float,
+    sa.VARCHAR: dt.String,
+    sa.Float: dt.Double,
+    sa.DATE: dt.Date,
 
     sa.types.TEXT: dt.String,
-    sa.types.NullType: dt.String,
+    sa.types.NullType: dt.Null,
     sa.types.Text: dt.String,
 }
 
@@ -84,8 +97,15 @@ def schema_from_table(table):
                 ibis_class = _sqla_type_to_ibis[c.type]
             elif type_class in _sqla_type_to_ibis:
                 ibis_class = _sqla_type_to_ibis[type_class]
+            elif isinstance(c.type, sa.DateTime):
+                ibis_class = dt.Timestamp()
             else:
-                raise NotImplementedError(c.type)
+                for k, v in _sqla_type_to_ibis.items():
+                    if isinstance(c.type, type(k)):
+                        ibis_class = v
+                        break
+                else:
+                    raise NotImplementedError(c.type)
             t = ibis_class(c.nullable)
 
         types.append(t)
@@ -392,14 +412,17 @@ class AlchemyContext(comp.QueryContext):
         self.dialect = kwargs.pop('dialect', AlchemyDialect)
         comp.QueryContext.__init__(self, *args, **kwargs)
 
-    def subcontext(self):
-        return type(self)(dialect=self.dialect, parent=self)
+    def subcontext(self, isolated=False):
+        if not isolated:
+            return type(self)(dialect=self.dialect, parent=self)
+        else:
+            return type(self)(dialect=self.dialect)
 
     def _to_sql(self, expr, ctx):
         return to_sqlalchemy(expr, context=ctx)
 
-    def _compile_subquery(self, expr):
-        sub_ctx = self.subcontext()
+    def _compile_subquery(self, expr, isolated=False):
+        sub_ctx = self.subcontext(isolated=isolated)
         return self._to_sql(expr, sub_ctx)
 
     def has_table(self, expr, parent_contexts=False):
@@ -770,6 +793,16 @@ class _AlchemyTableSet(TableSetFormatter):
 
         if isinstance(ref_op, AlchemyTable):
             result = ref_op.sqla_table
+        elif isinstance(ref_op, ops.UnboundTable):
+            # use SQLAlchemy's TableClause and ColumnClause for unbound tables
+            schema = ref_op.schema
+            result = sa.table(
+                ref_op.name if ref_op.name is not None else ctx.get_ref(expr),
+                *(
+                    sa.column(n, _to_sqla_type(t))
+                    for n, t in zip(schema.names, schema.types)
+                )
+            )
         else:
             # A subquery
             if ctx.is_extracted(ref_expr):
@@ -894,3 +927,61 @@ def _floor_divide(t, expr):
         return t.translate(new_expr)
 
     return fixed_arity(lambda x, y: x / y, 2)(t, expr)
+
+
+@compiles(ops.SortKey)
+def _sort_key(t, expr):
+    # We need to define this for window functions that have an order by
+    by, ascending = expr.op().args
+    sort_direction = sa.asc if ascending else sa.desc
+    return sort_direction(t.translate(by))
+
+
+_valid_frame_types = numbers.Integral, str, type(None)
+
+
+class Over(_Over):
+    def __init__(
+        self,
+        element,
+        order_by=None,
+        partition_by=None,
+        preceding=None,
+        following=None,
+    ):
+        super(Over, self).__init__(
+            element, order_by=order_by, partition_by=partition_by
+        )
+        if not isinstance(preceding, _valid_frame_types):
+            raise TypeError(
+                'preceding must be a string, integer or None, got %r' % (
+                    type(preceding).__name__
+                )
+            )
+        if not isinstance(following, _valid_frame_types):
+            raise TypeError(
+                'following must be a string, integer or None, got %r' % (
+                    type(following).__name__
+                )
+            )
+        self.preceding = preceding if preceding is not None else 'UNBOUNDED'
+        self.following = following if following is not None else 'UNBOUNDED'
+
+
+@sa_compiles(Over)
+def compile_over_with_frame(element, compiler, **kw):
+    clauses = ' '.join(
+        '%s BY %s' % (word, compiler.process(clause, **kw))
+        for word, clause in (
+            ('PARTITION', element.partition_by),
+            ('ORDER', element.order_by),
+        )
+        if clause is not None and len(clause)
+    )
+    return '%s OVER (%s%sROWS BETWEEN %s PRECEDING AND %s FOLLOWING)' % (
+        compiler.process(getattr(element, 'element', element.func), **kw),
+        clauses,
+        ' ' if clauses else '',  # only add a space if we order by or group by
+        str(element.preceding).upper(),
+        str(element.following).upper(),
+    )
