@@ -21,6 +21,7 @@ import ibis.expr.analytics as analytics
 
 import ibis.expr.operations as ops
 import ibis.expr.types as ir
+import ibis.expr.format as format
 
 import ibis.sql.transforms as transforms
 import ibis.util as util
@@ -147,12 +148,17 @@ class SelectBuilder(object):
         node = expr.op()
         if isinstance(node, ops.Join):
             for arg in node.args:
-                if not isinstance(arg, ir.TableExpr):
-                    continue
-                self._make_table_aliases(arg)
+                if isinstance(arg, ir.TableExpr):
+                    self._make_table_aliases(arg)
         else:
             if not ctx.is_extracted(expr):
                 ctx.make_alias(expr)
+            else:
+                # The compiler will apply a prefix only if the current context
+                # contains two or more table references. So, if we've extracted
+                # a subquery into a CTE, we need to propagate that reference
+                # down to child contexts so that they aren't missing any refs.
+                ctx.set_ref(expr, ctx.top_context.get_ref(expr))
 
     # ---------------------------------------------------------------------
     # Expr analysis / rewrites
@@ -913,7 +919,7 @@ class QueryContext(object):
     Records bits of information used during ibis AST to SQL translation
     """
 
-    def __init__(self, indent=2, parent=None):
+    def __init__(self, indent=2, parent=None, memo=None):
         self._table_refs = {}
         self.extracted_subexprs = set()
         self.subquery_memo = {}
@@ -925,9 +931,10 @@ class QueryContext(object):
         self.query = None
 
         self._table_key_memo = {}
+        self.memo = memo or format.FormatMemo()
 
-    def _compile_subquery(self, expr):
-        sub_ctx = self.subcontext()
+    def _compile_subquery(self, expr, isolated=False):
+        sub_ctx = self.subcontext(isolated=isolated)
         return self._to_sql(expr, sub_ctx)
 
     def _to_sql(self, expr, ctx):
@@ -943,7 +950,7 @@ class QueryContext(object):
     def set_always_alias(self):
         self.always_alias = True
 
-    def get_compiled_expr(self, expr):
+    def get_compiled_expr(self, expr, isolated=False):
         this = self.top_context
 
         key = self._get_table_key(expr)
@@ -954,7 +961,7 @@ class QueryContext(object):
         if isinstance(op, ops.SQLQueryResult):
             result = op.query
         else:
-            result = self._compile_subquery(expr)
+            result = self._compile_subquery(expr, isolated=isolated)
 
         this.subquery_memo[key] = result
         return result
@@ -1009,8 +1016,11 @@ class QueryContext(object):
         self.extracted_subexprs.add(key)
         self.make_alias(expr)
 
-    def subcontext(self):
-        return type(self)(indent=self.indent, parent=self)
+    def subcontext(self, isolated=False):
+        if not isolated:
+            return type(self)(indent=self.indent, parent=self)
+        else:
+            return type(self)(indent=self.indent)
 
     # Maybe temporary hacks for correlated / uncorrelated subqueries
 
@@ -1300,20 +1310,34 @@ class Select(DDL):
                                      permit_subquery=permit_subquery)
         return translator.get_result()
 
-    def equals(self, other):
+    def equals(self, other, cache=None):
+        if cache is None:
+            cache = {}
+
+        if (self, other) in cache:
+            return cache[(self, other)]
+
+        if id(self) == id(other):
+            cache[(self, other)] = True
+            return True
+
         if not isinstance(other, Select):
+            cache[(self, other)] = False
             return False
 
         this_exprs = self._all_exprs()
         other_exprs = other._all_exprs()
 
         if self.limit != other.limit:
+            cache[(self, other)] = False
             return False
 
         for x, y in zip(this_exprs, other_exprs):
             if not x.equals(y):
+                cache[(self, other)] = False
                 return False
 
+        cache[(self, other)] = True
         return True
 
     def _all_exprs(self):
