@@ -1447,6 +1447,39 @@ FROM t0
 GROUP BY 1"""
         assert result == expected
 
+    def test_subquery_in_union(self):
+        t = self.con.table('alltypes')
+
+        expr1 = t.group_by(['a', 'g']).aggregate(t.f.sum().name('metric'))
+        expr2 = expr1.view()
+
+        join1 = expr1.join(expr2, expr1.g == expr2.g)[[expr1]]
+        join2 = join1.view()
+
+        expr = join1.union(join2)
+        result = to_sql(expr)
+        expected = """\
+(WITH t0 AS (
+  SELECT `a`, `g`, sum(`f`) AS `metric`
+  FROM alltypes
+  GROUP BY 1, 2
+)
+SELECT t0.*
+FROM t0
+  INNER JOIN t0 t1
+    ON t0.`g` = t1.`g`)
+UNION ALL
+(WITH t0 AS (
+  SELECT `a`, `g`, sum(`f`) AS `metric`
+  FROM alltypes
+  GROUP BY 1, 2
+)
+SELECT t0.*
+FROM t0
+  INNER JOIN t0 t1
+    ON t0.`g` = t1.`g`)"""
+        assert result == expected
+
     def test_subquery_factor_correlated_subquery(self):
         # #173, #183 and other issues
 
@@ -2235,3 +2268,55 @@ SELECT `string_col`, COUNT(DISTINCT `int_col`) AS `int_card`,
 FROM functional_alltypes
 GROUP BY 1"""
         assert result == expected
+
+
+def test_pushdown_with_or():
+    t = ibis.table(
+        [('double_col', 'double'),
+         ('string_col', 'string'),
+         ('int_col', 'int32'),
+         ('float_col', 'float')],
+        'functional_alltypes',
+    )
+    subset = t[(t.double_col > 3.14) & t.string_col.contains('foo')]
+    filt = subset[(subset.int_col - 1 == 0) | (subset.float_col <= 1.34)]
+    result = to_sql(filt)
+    expected = """\
+SELECT *
+FROM functional_alltypes
+WHERE (`double_col` > 3.14) AND (locate('foo', `string_col`) - 1 >= 0) AND
+      (((`int_col` - 1) = 0) OR (`float_col` <= 1.34))"""
+    assert result == expected
+
+
+def test_having_size():
+    t = ibis.table(
+        [('double_col', 'double'),
+         ('string_col', 'string'),
+         ('int_col', 'int32'),
+         ('float_col', 'float')],
+        'functional_alltypes',
+    )
+    expr = t.group_by(t.string_col).having(t.double_col.max() == 1).size()
+    result = to_sql(expr)
+    assert result == """\
+SELECT `string_col`, count(*) AS `count`
+FROM functional_alltypes
+GROUP BY 1
+HAVING max(`double_col`) = 1"""
+
+
+def test_having_from_filter():
+    t = ibis.table([('a', 'int64'), ('b', 'string')], 't')
+    filt = t[t.b == 'm']
+    gb = filt.group_by(filt.b)
+    having = gb.having(filt.a.max() == 2)
+    agg = having.aggregate(filt.a.sum().name('sum'))
+    result = to_sql(agg)
+    expected = """\
+SELECT `b`, sum(`a`) AS `sum`
+FROM t
+WHERE `b` = 'm'
+GROUP BY 1
+HAVING max(`a`) = 2"""
+    assert result == expected
